import { __decorate, __metadata, __param } from "tslib";
import { I18N } from '@aurelia/i18n';
import { optional } from 'aurelia';
/**
 * This is a Translate Service Wrapper for Slickgrid-Universal monorepo lib to work properly,
 * it must implement Slickgrid-Universal TranslaterService interface to work properly
 */
let TranslaterService = class TranslaterService {
    i18n;
    constructor(i18n) {
        this.i18n = i18n;
    }
    /**
     * Method to return the current language used by the App
     * @return {string} current language
     */
    getCurrentLanguage() {
        return this.i18n?.getLocale();
    }
    /**
     * Method to set the language to use in the App and Translate Service
     * @param {string} language
     * @return {Promise} output
     */
    async use(newLang) {
        return this.i18n?.setLocale(newLang);
    }
    /**
     * Method which receives a translation key and returns the translated value assigned to that key
     * @param {string} translation key
     * @return {string} translated value
     */
    translate(translationKey) {
        return this.i18n?.tr(translationKey);
    }
};
TranslaterService = __decorate([
    __param(0, optional(I18N)),
    __param(0, I18N),
    __metadata("design:paramtypes", [Object])
], TranslaterService);
export { TranslaterService };
//# sourceMappingURL=translater.service.js.map