import { EventSubscription, OnBeforeRowDetailToggleArgs, OnRowBackToViewportRangeArgs, SlickEventData, SlickEventHandler, SlickGrid, SlickRowSelectionModel } from '@slickgrid-universal/common';
import { EventPubSubService } from '@slickgrid-universal/event-pub-sub';
import { SlickRowDetailView as UniversalSlickRowDetailView } from '@slickgrid-universal/row-detail-view-plugin';
import { Constructable } from 'aurelia';
import { AureliaViewOutput, GridOption, RowDetailView } from '../models/index';
import { AureliaUtilService } from '../services/aureliaUtil.service';
export interface CreatedView extends AureliaViewOutput {
    id: string | number;
    dataContext: any;
}
export declare class SlickRowDetailView extends UniversalSlickRowDetailView {
    protected readonly aureliaUtilService: AureliaUtilService;
    private readonly eventPubSubService;
    private readonly gridContainerElement;
    protected _preloadViewModel?: Constructable;
    protected _slots: CreatedView[];
    protected _viewModel?: Constructable;
    protected _subscriptions: EventSubscription[];
    protected _userProcessFn?: (item: any) => Promise<any>;
    constructor(aureliaUtilService: AureliaUtilService, eventPubSubService: EventPubSubService, gridContainerElement: HTMLElement);
    get addonOptions(): import("@slickgrid-universal/common").RowDetailViewOption;
    protected get datasetIdPropName(): string;
    get eventHandler(): SlickEventHandler;
    set eventHandler(eventHandler: SlickEventHandler);
    /** Getter for the Grid Options pulled through the Grid Object */
    get gridOptions(): GridOption;
    get rowDetailViewOptions(): RowDetailView | undefined;
    /** Dispose of the RowDetailView Extension */
    dispose(): void;
    /** Dispose of all the opened Row Detail Panels Aurelia View Slots */
    disposeAllViewSlot(): void;
    /** Get the instance of the SlickGrid addon (control or plugin). */
    getAddonInstance(): SlickRowDetailView | null;
    init(grid: SlickGrid): void;
    /**
     * Create the plugin before the Grid creation, else it will behave oddly.
     * Mostly because the column definitions might change after the grid creation
     */
    register(rowSelectionPlugin?: SlickRowSelectionModel): this;
    /** Redraw (re-render) all the expanded row detail View Slots */
    redrawAllViewSlots(): Promise<void>;
    /** Render all the expanded row detail View Slots */
    renderAllViewModels(): Promise<void>;
    /** Redraw the necessary View Slot */
    redrawViewSlot(slot: CreatedView): Promise<void>;
    /** Render (or re-render) the View Slot (Row Detail) */
    renderPreloadView(): Promise<void>;
    /** Render (or re-render) the View Slot (Row Detail) */
    renderViewModel(item: any): Promise<void>;
    protected disposeViewSlot(expandedView: CreatedView): CreatedView | void;
    /**
     * Just before the row get expanded or collapsed we will do the following
     * First determine if the row is expanding or collapsing,
     * if it's expanding we will add it to our View Slots reference array if we don't already have it
     * or if it's collapsing we will remove it from our View Slots reference array
     */
    protected handleOnBeforeRowDetailToggle(_e: SlickEventData<OnBeforeRowDetailToggleArgs>, args: {
        grid: SlickGrid;
        item: any;
    }): void;
    /** When Row comes back to Viewport Range, we need to redraw the View */
    protected handleOnRowBackToViewportRange(_e: SlickEventData<OnRowBackToViewportRangeArgs>, args: {
        item: any;
        rowId: string | number;
        rowIndex: number;
        expandedRows: (string | number)[];
        rowIdsOutOfViewport: (string | number)[];
        grid: SlickGrid;
    }): Promise<void>;
    /**
     * notify the onAsyncResponse with the "args.item" (required property)
     * the plugin will then use item to populate the row detail panel with the "postTemplate"
     * @param item
     */
    protected notifyTemplate(item: any): void;
    /**
     * On Processing, we will notify the plugin with the new item detail once backend server call completes
     * @param item
     */
    protected onProcessing(item: any): Promise<void>;
}
